@@ -35,7 +35,11 @@ class ActorTexts:
         self._misc_texts = {}
         self._actor_name = pack.stem
         self._profile = profile
-        root_dir = Path(str(pack).split("Actor")[0])
+        root_dir = pack.parent
+        while True:
+            if root_dir.name == "content":
+                break
+            root_dir = root_dir.parent
         settings = util.BatSettings()
         lang = settings.get_setting("lang")
         text_pack = root_dir / f"Pack/Bootup_{lang}.pack"
@@ -77,6 +81,10 @@ class ActorTexts:
 
     def write(self, root_str: str, be: bool) -> None:
         if self._texts:
+            for entry, text in self._texts.items():
+                entry_name = f"{self._actor_name}_{entry}"
+                self._misc_texts[entry_name] = {"contents": [{"text": text}]}  # type:ignore[index]
+
             settings = util.BatSettings()
             msyt = {
                 "group_count": self._group_count,
@@ -85,9 +93,6 @@ class ActorTexts:
             }
             for entry, data in self._misc_texts.items():
                 msyt["entries"][entry] = data  # type:ignore[index]
-            for entry, text in self._texts.items():
-                entry_name = f"{self._actor_name}_{entry}"
-                msyt["entries"][entry_name] = {"contents": [{"text": text}]}  # type:ignore[index]
             platform = "wiiu" if be else "switch"
             temp = settings.get_data_dir() / "temp.msbt"
             pymsyt.write_msbt(msyt, temp, platform=platform)
